@@ -306,7 +306,7 @@ bool QAlsaAudioOutput::open()
     QString dev = QString(QLatin1String(m_device.constData()));
     QList<QByteArray> devices = QAlsaAudioDeviceInfo::availableDevices(QAudio::AudioOutput);
     if(dev.compare(QLatin1String("default")) == 0) {
-#if(SND_LIB_MAJOR == 1 && SND_LIB_MINOR == 0 && SND_LIB_SUBMINOR >= 14)
+#if (SND_LIB_MAJOR == 1 && (SND_LIB_MINOR > 0 || SND_LIB_SUBMINOR >= 14))
         if (devices.size() > 0)
             dev = QLatin1String(devices.first());
         else
@@ -315,7 +315,7 @@ bool QAlsaAudioOutput::open()
         dev = QLatin1String("hw:0,0");
 #endif
     } else {
-#if(SND_LIB_MAJOR == 1 && SND_LIB_MINOR == 0 && SND_LIB_SUBMINOR >= 14)
+#if (SND_LIB_MAJOR == 1 && (SND_LIB_MINOR > 0 || SND_LIB_SUBMINOR >= 14))
         dev = QLatin1String(m_device);
 #else
         int idx = 0;
@@ -651,7 +651,7 @@ void QAlsaAudioOutput::resume()
         }
         resuming = true;
 
-        deviceState = QAudio::ActiveState;
+        deviceState = pullMode ? QAudio::ActiveState : QAudio::IdleState;
 
         errorState = QAudio::NoError;
         timer->start(period_time/1000);
