@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -78,8 +78,8 @@ AndroidSurfaceTexture::AndroidSurfaceTexture(unsigned int texName)
 
 AndroidSurfaceTexture::~AndroidSurfaceTexture()
 {
-    if (QtAndroidPrivate::androidSdkVersion() > 13 && m_surfaceView.isValid())
-        m_surfaceView.callMethod<void>("release");
+    if (QtAndroidPrivate::androidSdkVersion() > 13 && m_surface.isValid())
+        m_surface.callMethod<void>("release");
 
     if (m_surfaceTexture.isValid()) {
         release();
@@ -124,21 +124,23 @@ jobject AndroidSurfaceTexture::surfaceTexture()
     return m_surfaceTexture.object();
 }
 
-jobject AndroidSurfaceTexture::surfaceView()
+jobject AndroidSurfaceTexture::surface()
 {
-    return m_surfaceView.object();
+    if (!m_surface.isValid()) {
+        m_surface = QJNIObjectPrivate("android/view/Surface",
+                                      "(Landroid/graphics/SurfaceTexture;)V",
+                                      m_surfaceTexture.object());
+    }
+
+    return m_surface.object();
 }
 
 jobject AndroidSurfaceTexture::surfaceHolder()
 {
     if (!m_surfaceHolder.isValid()) {
-        m_surfaceView = QJNIObjectPrivate("android/view/Surface",
-                                          "(Landroid/graphics/SurfaceTexture;)V",
-                                          m_surfaceTexture.object());
-
         m_surfaceHolder = QJNIObjectPrivate("org/qtproject/qt5/android/multimedia/QtSurfaceTextureHolder",
                                             "(Landroid/view/Surface;)V",
-                                            m_surfaceView.object());
+                                            surface());
     }
 
     return m_surfaceHolder.object();
